@@ -26,7 +26,8 @@ from io import StringIO
 from enum import Enum
 from glob import glob
 from pathlib import Path
-import mesonbuild
+from mesonbuild import compilers
+from mesonbuild import dependencies
 from mesonbuild import mesonlib
 from mesonbuild import mesonmain
 from mesonbuild import mtest
@@ -34,11 +35,13 @@ from mesonbuild import mlog
 from mesonbuild.environment import Environment, detect_ninja
 from mesonbuild.coredata import backendlist
 
-def guess_backend(backend, msbuild_exe):
+NINJA_1_9_OR_NEWER = False
+
+def guess_backend(backend, msbuild_exe: str):
     # Auto-detect backend if unspecified
     backend_flags = []
     if backend is None:
-        if msbuild_exe is not None and mesonlib.is_windows():
+        if msbuild_exe is not None and (mesonlib.is_windows() and not _using_intelcl()):
             backend = 'vs' # Meson will auto-detect VS version to use
         else:
             backend = 'ninja'
@@ -57,6 +60,32 @@ def guess_backend(backend, msbuild_exe):
     return (backend, backend_flags)
 
 
+def _using_intelcl() -> bool:
+    """
+    detect if intending to using Intel-Cl compilers (Intel compilers on Windows)
+    Sufficient evidence of intent is that user is working in the Intel compiler
+    shell environment, otherwise this function returns False
+    """
+    if not mesonlib.is_windows():
+        return False
+    # handle where user tried to "blank" MKLROOT and left space(s)
+    if not os.environ.get('MKLROOT', '').strip():
+        return False
+    if (os.environ.get('CC') == 'icl' or
+            os.environ.get('CXX') == 'icl' or
+            os.environ.get('FC') == 'ifort'):
+        return True
+    # Intel-Cl users might not have the CC,CXX,FC envvars set,
+    # but because they're in Intel shell, the exe's below are on PATH
+    if shutil.which('icl') or shutil.which('ifort'):
+        return True
+    mlog.warning('It appears you might be intending to use Intel compiler on Windows '
+                 'since non-empty environment variable MKLROOT is set to {} '
+                 'However, Meson cannot find the Intel WIndows compiler executables (icl,ifort).'
+                 'Please try using the Intel shell.'.format(os.environ.get('MKLROOT')))
+    return False
+
+
 # Fake classes and objects for mocking
 class FakeBuild:
     def __init__(self, env):
@@ -67,7 +96,6 @@ class FakeCompilerOptions:
         self.value = []
 
 def get_fake_options(prefix=''):
-    import argparse
     opts = argparse.Namespace()
     opts.native_file = []
     opts.cross_file = None
@@ -88,8 +116,7 @@ def get_fake_env(sdir='', bdir=None, prefix='', opts=None):
 Backend = Enum('Backend', 'ninja vs xcode')
 
 if 'MESON_EXE' in os.environ:
-    import shlex
-    meson_exe = shlex.split(os.environ['MESON_EXE'])
+    meson_exe = mesonlib.split_args(os.environ['MESON_EXE'])
 else:
     meson_exe = None
 
@@ -108,7 +135,7 @@ def get_meson_script():
     running in-process (which is the default).
     '''
     # Is there a meson.py next to the mesonbuild currently in use?
-    mesonbuild_dir = Path(mesonbuild.__file__).resolve().parent.parent
+    mesonbuild_dir = Path(mesonmain.__file__).resolve().parent.parent
     meson_script = mesonbuild_dir / 'meson.py'
     if meson_script.is_file():
         return str(meson_script)
@@ -175,8 +202,20 @@ def get_backend_commands(backend, debug=False):
         clean_cmd = cmd + ['-alltargets', 'clean', '-UseNewBuildSystem=FALSE']
         test_cmd = cmd + ['-target', 'RUN_TESTS']
     elif backend is Backend.ninja:
-        # We need at least 1.6 because of -w dupbuild=err
-        cmd = [detect_ninja('1.6'), '-w', 'dupbuild=err', '-d', 'explain']
+        global NINJA_1_9_OR_NEWER
+        # Look for 1.9 to see if https://github.com/ninja-build/ninja/issues/1219
+        # is fixed, else require 1.6 for -w dupbuild=err
+        for v in ('1.9', '1.6'):
+            ninja_cmd = detect_ninja(v)
+            if ninja_cmd is not None:
+                if v == '1.9':
+                    NINJA_1_9_OR_NEWER = True
+                else:
+                    mlog.warning('Found ninja <1.9, tests will run slower', once=True)
+                    if 'CI' in os.environ:
+                        raise RuntimeError('Require ninja >= 1.9 when running on Meson CI')
+                break
+        cmd = [ninja_cmd, '-w', 'dupbuild=err', '-d', 'explain']
         if cmd[0] is None:
             raise RuntimeError('Could not find Ninja v1.6 or newer')
         if debug:
@@ -190,16 +229,25 @@ def get_backend_commands(backend, debug=False):
     return cmd, clean_cmd, test_cmd, install_cmd, uninstall_cmd
 
 def ensure_backend_detects_changes(backend):
-    # We're using a ninja with QuLogic's patch for sub-1s resolution timestamps
-    # and not running on HFS+ which only stores dates in seconds:
-    # https://developer.apple.com/legacy/library/technotes/tn/tn1150.html#HFSPlusDates
-    # FIXME: Upgrade Travis image to Apple FS when that becomes available
-    if 'MESON_FIXED_NINJA' in os.environ and not mesonlib.is_osx():
+    global NINJA_1_9_OR_NEWER
+    if backend is not Backend.ninja:
         return
-    # This is needed to increase the difference between build.ninja's
-    # timestamp and the timestamp of whatever you changed due to a Ninja
-    # bug: https://github.com/ninja-build/ninja/issues/371
-    if backend is Backend.ninja:
+    need_workaround = False
+    # We're not running on HFS+ which only stores dates in seconds:
+    # https://developer.apple.com/legacy/library/technotes/tn/tn1150.html#HFSPlusDates
+    # XXX: Upgrade Travis image to Apple FS when that becomes available
+    # TODO: Detect HFS+ vs APFS
+    if mesonlib.is_osx():
+        mlog.warning('Running on HFS+, enabling timestamp resolution workaround', once=True)
+        need_workaround = True
+    # We're using ninja >= 1.9 which has QuLogic's patch for sub-1s resolution
+    # timestamps
+    if not NINJA_1_9_OR_NEWER:
+        mlog.warning('Don\'t have ninja >= 1.9, enabling timestamp resolution workaround', once=True)
+        need_workaround = True
+    # Increase the difference between build.ninja's timestamp and the timestamp
+    # of whatever you changed: https://github.com/ninja-build/ninja/issues/371
+    if need_workaround:
         time.sleep(1)
 
 def run_mtest_inprocess(commandlist):
@@ -215,12 +263,12 @@ def run_mtest_inprocess(commandlist):
     return returncode, mystdout.getvalue(), mystderr.getvalue()
 
 def clear_meson_configure_class_caches():
-    mesonbuild.compilers.CCompiler.library_dirs_cache = {}
-    mesonbuild.compilers.CCompiler.program_dirs_cache = {}
-    mesonbuild.compilers.CCompiler.find_library_cache = {}
-    mesonbuild.compilers.CCompiler.find_framework_cache = {}
-    mesonbuild.dependencies.PkgConfigDependency.pkgbin_cache = {}
-    mesonbuild.dependencies.PkgConfigDependency.class_pkgbin = mesonlib.PerMachine(None, None)
+    compilers.CCompiler.library_dirs_cache = {}
+    compilers.CCompiler.program_dirs_cache = {}
+    compilers.CCompiler.find_library_cache = {}
+    compilers.CCompiler.find_framework_cache = {}
+    dependencies.PkgConfigDependency.pkgbin_cache = {}
+    dependencies.PkgConfigDependency.class_pkgbin = mesonlib.PerMachine(None, None)
 
 def run_configure_inprocess(commandlist, env=None):
     old_stdout = sys.stdout
@@ -267,6 +315,7 @@ def main():
                         choices=backendlist)
     parser.add_argument('--cross', default=False, dest='cross', action='store_true')
     parser.add_argument('--failfast', action='store_true')
+    parser.add_argument('--no-unittests', action='store_true', default=False)
     (options, _) = parser.parse_known_args()
     # Enable coverage early...
     enable_coverage = options.cov
@@ -278,6 +327,7 @@ def main():
     returncode = 0
     cross = options.cross
     backend, _ = guess_backend(options.backend, shutil.which('msbuild'))
+    no_unittests = options.no_unittests
     # Running on a developer machine? Be nice!
     if not mesonlib.is_windows() and not mesonlib.is_haiku() and 'CI' not in os.environ:
         os.nice(20)
@@ -319,12 +369,16 @@ def main():
             returncode += subprocess.call(cmd, env=env)
             if options.failfast and returncode != 0:
                 return returncode
-            cmd = mesonlib.python_command + ['run_unittests.py', '-v']
-            if options.failfast:
-                cmd += ['--failfast']
-            returncode += subprocess.call(cmd, env=env)
-            if options.failfast and returncode != 0:
-                return returncode
+            if no_unittests:
+                print('Skipping all unit tests.')
+                returncode = 0
+            else:
+                cmd = mesonlib.python_command + ['run_unittests.py', '-v']
+                if options.failfast:
+                    cmd += ['--failfast']
+                returncode += subprocess.call(cmd, env=env)
+                if options.failfast and returncode != 0:
+                    return returncode
             cmd = mesonlib.python_command + ['run_project_tests.py'] + sys.argv[1:]
             returncode += subprocess.call(cmd, env=env)
         else:
